package org.thibault.android.clock.binary;

import java.util.Calendar;
import java.util.TimeZone;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Canvas;
import android.graphics.Color;
import android.os.Handler;
import android.util.Log;
import android.view.View;

public class Clock extends View {
	private final Context context;
	private Style style;
	private boolean withText, paused = false;
	private void say(String msg){
		Log.d("BinaryClock.Clock",msg);
		//Toast t=Toast.makeText(context,msg,Toast.LENGTH_SHORT);
		//t.show();
	}

	public Clock(Context _context) {
		super(_context);
		context=_context;
		setStyle(StyleRegistry.BLACK_ON_WHITE,false);
	}
	
	public void setStyle(int styleId, boolean _withText) {
		style=new StyleRegistry().getStyle(styleId, context.getResources());
		withText=_withText;
		invalidate();
	}

    private final static IntentFilter timeIntentFilter;
    static {
        timeIntentFilter = new IntentFilter();
        timeIntentFilter.addAction(Intent.ACTION_TIME_TICK);
        timeIntentFilter.addAction(Intent.ACTION_TIMEZONE_CHANGED);
        timeIntentFilter.addAction(Intent.ACTION_TIME_CHANGED);
    }
    
    private BroadcastReceiver timeInfoReceiver = null;
    
    public void monitorTime(Activity activity) {
    	if (timeInfoReceiver != null)
    		return;
    	
        timeInfoReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context arg0, Intent intent) {
                if (!paused)
                    invalidate();
            }
        };

        activity.registerReceiver(timeInfoReceiver, timeIntentFilter);
    }
    
    public void stopMonitoringTime(Activity activity) {
    	if (timeIntentFilter != null) {
    		activity.unregisterReceiver(timeInfoReceiver);
    		timeInfoReceiver = null;
    	}
    }

    @Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		final int tick=Updater.getTick();
		final int diameter=Updater.min(getWidth(),getHeight());
		//say("onDraw(): tick=="+tick+"; diameter=="+diameter+"; width=="+getWidth()+"; height=="+getHeight());
		style.drawClock(tick, diameter, withText, canvas);
	}
	
    public void pause() {paused = true;}
    public void resume() {
        paused = false;
        invalidate();
    }
}