package org.thibault.android.clock.binary;

import android.app.Activity;
import android.appwidget.AppWidgetManager;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.Spinner;

public class Configure extends Activity {

	public static final String RECONFIGURING="RECONFIGURING";
	private int widgetId=AppWidgetManager.INVALID_APPWIDGET_ID;
	private boolean reconfiguring=false;
	private Clock clock = null;
	
	private void say(String msg){
		//Log.d("BinaryClock.Configure",msg);
		//Toast t=Toast.makeText(context,msg,Toast.LENGTH_SHORT);
		//t.show();
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		//Context context=getApplicationContext();
		super.onCreate(savedInstanceState);
		{
			Intent resultValue = new Intent();
			setResult(RESULT_CANCELED, resultValue);
		}

		setContentView(R.layout.configure);
		
		Intent intent = getIntent();
		Bundle extras = intent.getExtras();
		if (extras == null) return;
		widgetId = extras.getInt(AppWidgetManager.EXTRA_APPWIDGET_ID, 
								 AppWidgetManager.INVALID_APPWIDGET_ID);
		
		say("widgetId=="+widgetId);
		
		if (widgetId==AppWidgetManager.INVALID_APPWIDGET_ID) {
			say("that's invalid");
			return;
		}

		reconfiguring=intent.getBooleanExtra(Configure.RECONFIGURING, false);
		say("reconfiguring=="+reconfiguring);
		
		{
			Intent resultValue = new Intent();
			resultValue.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, widgetId);
			setResult(RESULT_CANCELED, resultValue);
		}
		
		SharedPreferences p=PreferenceManager.getDefaultSharedPreferences(this);
		Updater.WidgetProperties prop = Updater.WidgetProperties.load(p, widgetId);
		int styleId;
		boolean withText;
		
		if (prop!=null) {
			styleId=prop.styleId;
			withText=prop.withText;
		} else {
			styleId=StyleRegistry.DEFAULT;
			withText=false;
		}

		Button saveButton=(Button)findViewById(R.id.saveButton);
		saveButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {save();}
		});

		Button helpButton=(Button)findViewById(R.id.helpButton);
		helpButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {help();}
		});

		Button openSourceButton=(Button)findViewById(R.id.openSourceButton);
		openSourceButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {openSource();}
		});

		final CheckBox withTextCheckBox=(CheckBox)findViewById(R.id.showTextClockCheckBox);

		ViewGroup container=(ViewGroup)findViewById(R.id.configureLayout);
		clock=new Clock(getApplicationContext());
        LinearLayout.LayoutParams params
    	=new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,
    								   LinearLayout.LayoutParams.MATCH_PARENT);
		container.addView(clock,params);
		clock.setStyle(styleId,withText);
		clock.monitorTime(this);
		
		final Spinner clockColorSpinner
		=(Spinner)findViewById(R.id.clockColorSpinner);
		ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
		        R.array.color_choices, android.R.layout.simple_spinner_item);
		clockColorSpinner.setAdapter(adapter);
		clockColorSpinner.setOnItemSelectedListener(new OnItemSelectedListener() {

			@Override
			public void onItemSelected(AdapterView<?> parent, View view,
					int pos, long id) {
				final int styleId = posToStyleId(pos);
				clock.setStyle(styleId,withTextCheckBox.isChecked());
			}

			@Override
			public void onNothingSelected(AdapterView<?> parent) {
				// Can't happen?
				clockColorSpinner.setSelection(posToStyleId(StyleRegistry.DEFAULT));
			}
			
		});
		
		withTextCheckBox.setOnCheckedChangeListener(new CheckBox.OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(CompoundButton buttonView,
										 boolean withText) {
				final int styleId
				=posToStyleId(clockColorSpinner.getSelectedItemPosition());
				clock.setStyle(styleId,withText);
			}
			
		});
		
		clockColorSpinner.setSelection(styleIdToPos(styleId));
		withTextCheckBox.setChecked(withText);
	}
	
	public void onBackPressed() {
		super.onBackPressed();
		if (reconfiguring)
			startActivity(new Intent(Intent.ACTION_MAIN).addCategory(Intent.CATEGORY_HOME));
	}
	
	private int styleIdToPos(int styleId) {
		switch (styleId) {
		case StyleRegistry.BLUE_ON_RED: return 0;
		case StyleRegistry.GREEN_ON_RED: return 1;
		case StyleRegistry.BLACK_ON_WHITE: return 2;
		case StyleRegistry.SAPPHIRE_ON_EMERALD: return 3;
		case StyleRegistry.SAPPHIRE_ON_WOOD: return 4;
		case StyleRegistry.EMERALD_ON_WOOD: return 5;
		default: return styleIdToPos(StyleRegistry.DEFAULT);
		}
	}
	
	private int posToStyleId(int checkedId) {
		switch (checkedId) {
		case 0: return StyleRegistry.BLUE_ON_RED;
		case 1: return StyleRegistry.GREEN_ON_RED;
		case 2: return StyleRegistry.BLACK_ON_WHITE;
		case 3: return StyleRegistry.SAPPHIRE_ON_EMERALD;
		case 4: return StyleRegistry.SAPPHIRE_ON_WOOD;
		case 5: return StyleRegistry.EMERALD_ON_WOOD;
		default: return StyleRegistry.DEFAULT;
		}
	}
	
	public void save() {
		if (widgetId==AppWidgetManager.INVALID_APPWIDGET_ID) // can't happen
			return;

		say("save()");
		
		AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(this);
		int[] appWidgetIds=new int[1];
		appWidgetIds[0]=widgetId;
		
		final CheckBox withTextCheckBox=(CheckBox)findViewById(R.id.showTextClockCheckBox);
		final boolean withText=withTextCheckBox.isChecked();

		final Spinner clockColorSpinner
		=(Spinner)findViewById(R.id.clockColorSpinner);
		final int styleId
		=posToStyleId(clockColorSpinner.getSelectedItemPosition());
		say("withText: "+withText);
		
		Widget._onUpdate(getApplicationContext(), appWidgetManager, appWidgetIds,
						 withText, styleId);
		Intent resultValue = new Intent();
		resultValue.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, widgetId);
		setResult(RESULT_OK, resultValue);
		if (reconfiguring)
			startActivity(new Intent(Intent.ACTION_MAIN).addCategory(Intent.CATEGORY_HOME));
		finish();
	}
	
	public void help() {
		startActivity(new Intent(this,Explanation.class));
	}

	public void openSource() {
    	Intent intent=new Intent();
    	intent.setAction(Intent.ACTION_VIEW);
    	intent.setData(Uri.parse(getResources().getString(R.string.ossURL)));
    	intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    	startActivity(intent);
	}
	
	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (clock != null)
			clock.stopMonitoringTime(this);
	}
}
