package org.thibault.android.clock.binary;

import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;

public class FlatTexter extends Texter {
	private final int foreColor;
	private final Paint backPaint;
	
	public FlatTexter(int _foreColor, int backColor, boolean backColorValid) {
		foreColor=_foreColor;
		if (backColorValid) {
			backPaint=new Paint();
			backPaint.setColor(backColor);
		} else {
			backPaint=null;
		}
	}

	@Override
	public void drawText(int tick, int diameter, Canvas c) {
		Paint textPaint=mkPaint(foreColor,Typeface.DEFAULT_BOLD,16,diameter);
		final int radius=diameter/2;
		int hour=tick/60;
		if (hour==0) hour=12;
		int min=tick%60;
		String time=""+hour+":"+((min<10) ? "0" : "")+min;
		
		Rect bounds=new Rect();
		textPaint.getTextBounds(time, 0, time.indexOf(':'), bounds);
		final int hourWidth=bounds.width();
		textPaint.getTextBounds(time, 0, time.indexOf(':')+1, bounds);
		final int colonWidth=bounds.width();
		final int colonCenter=(hourWidth+colonWidth)/2;
		textPaint.getTextBounds(time, 0, time.length(), bounds);
		
		final int textLeft=radius-colonCenter,textBottom=diameter-bounds.height()/2,
				textRight=textLeft+bounds.width(),textTop=textBottom-bounds.height();
		if (backPaint!=null)
			c.drawRect(textLeft-2, textBottom+2, textRight+2, textTop-2, backPaint);
		
		c.drawText(time, radius-colonCenter, textBottom, textPaint);
	}
}