package org.thibault.android.clock.binary;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Path;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.Region;

public abstract class Style {

	protected final Filler foreFiller, backFiller;
	protected final Texter texter;
	private Texter backTexter;
	protected final Separator separator;
	
	protected Texter getBackTexter() {return backTexter;}
	
	protected Style(Filler _foreFiller, Filler _backFiller, Texter _texter, Separator _separator) {
		foreFiller=_foreFiller;
		backFiller=_backFiller;
		texter=_texter;
		separator=_separator;
		backTexter=null;
	}

	public void drawClock(final int tick, final int diameter,
						  boolean withText, Canvas c) {
		final int radius=diameter/2;
		
		Rect fullSize=new Rect(0,0,diameter,diameter);
		c.clipRect(fullSize,Region.Op.REPLACE);
		backFiller.fillCircle(radius, radius, radius, c);
		
		c.clipRect(new Rect(0,0,0,0),Region.Op.REPLACE);
		
		for (int bit=9; bit>=5; bit--) {
			if ((tick&(1<<bit))!=0) {
				int th=72*bit-90;
				Path path = new Path();
				RectF rect=new RectF(0,0,diameter,diameter);
				float thRad=(float)(th*Math.PI/180);
				path.moveTo(radius,radius);
				path.lineTo((float)(radius+radius*Math.cos(thRad)),(float)(radius+radius*Math.sin(thRad)));
				path.arcTo(rect,th,72);
				path.lineTo(radius, radius);
				path.close();
				c.clipPath(path, Region.Op.UNION);
			}
		}
		
		foreFiller.fillCircle(radius, radius, radius, c);
		{
			Path path=new Path();
			path.addCircle(radius, radius, radius/2, Path.Direction.CW);
			path.close();
			c.clipPath(path,Region.Op.REPLACE);
		}
		backFiller.fillCircle(radius, radius, radius, c);

		c.clipRect(new Rect(0,0,0,0),Region.Op.REPLACE);
		for (int bit=4; bit>=0; bit--) {
			if ((tick&(1<<bit))!=0) {
				int th=72*bit-90;
				int r=radius/2;
				Path path = new Path();
				RectF rect=new RectF(radius-r,radius-r,radius+r,radius+r);
				float thRad=(float)(th*Math.PI/180);
				path.moveTo(radius,radius);
				path.lineTo((float)(radius+r*Math.cos(thRad)),(float)(radius+r*Math.sin(thRad)));
				path.arcTo(rect,th,72);
				path.lineTo(radius, radius);
				path.close();
				c.clipPath(path, Region.Op.UNION);
			}
		}
		foreFiller.fillCircle(radius, radius, radius, c);
		c.clipRect(fullSize,Region.Op.REPLACE);

		separator.drawSeparations(diameter, c);
		
		if (withText) {
			final Texter t=(((backTexter!=null) && ((tick & 128)==0)) ? backTexter : texter);
			t.drawText(tick, diameter, c);
		}
	}
	
	void setBackgroundTexter(Texter _backTexter) {backTexter=_backTexter;}
}
