package org.thibault.android.clock.binary;

import android.content.res.Resources;
import android.graphics.Color;

public class StyleRegistry {

	private Style blueOnRed, sapphireOnEmerald, greenOnRed, blackOnWhite, sapphireOnWood, emeraldOnWood;
	
	public static final int BLUE_ON_RED=1,
			           		SAPPHIRE_ON_EMERALD=2,
			           		GREEN_ON_RED=3,
			           		BLACK_ON_WHITE=4,
			           		SAPPHIRE_ON_WOOD=5,
			           		EMERALD_ON_WOOD=6,
			           		DEFAULT=SAPPHIRE_ON_EMERALD;
	
	public synchronized Style getStyle(int id, Resources resources) {
		switch (id) {
		case BLUE_ON_RED:
			if (blueOnRed==null)
				blueOnRed=mkBlueOnRed();
			return blueOnRed;
			
		case GREEN_ON_RED:
			if (greenOnRed==null)
				greenOnRed=mkGreenOnRed();
			return greenOnRed;
			
		case SAPPHIRE_ON_EMERALD:
			if (sapphireOnEmerald==null)
				sapphireOnEmerald=mkSapphireOnEmerald(resources);
			return sapphireOnEmerald;
			
		case SAPPHIRE_ON_WOOD:
			if (sapphireOnWood==null)
				sapphireOnWood=mkSapphireOnWood(resources);
			return sapphireOnWood;
			
		case EMERALD_ON_WOOD:
			if (emeraldOnWood==null)
				emeraldOnWood=mkEmeraldOnWood(resources);
			return emeraldOnWood;
			
		case BLACK_ON_WHITE:
		default:
			if (blackOnWhite==null)
				blackOnWhite=mkBlackOnWhite();
			return blackOnWhite;
		}
	}
	
	private static final int darkBrown = Color.rgb(122,16,16);
	
	private Style mkBlueOnRed() {
		return new FlatStyle(Color.BLUE,Color.RED,Color.BLACK);
	}
	private Style mkGreenOnRed() {
		return new FlatStyle(Color.rgb(0, 0xc8, 0x19),Color.RED,Color.BLACK);
	}
	private Style mkSapphireOnEmerald(Resources resources) {
		return new DrawableStyle(R.drawable.stone2, R.drawable.stone1, Color.YELLOW, -1, false, darkBrown, resources);
	}
	private Style mkSapphireOnWood(Resources resources) {
		Style res=new DrawableStyle(R.drawable.stone2, R.drawable.wood, Color.WHITE, -1, false, darkBrown, resources);
		res.setBackgroundTexter(new FlatTexter(Color.BLACK,-1,false));
		return res;
	}
	private Style mkEmeraldOnWood(Resources resources) {
		return new DrawableStyle(R.drawable.stone1, R.drawable.wood, Color.BLACK, -1, false, darkBrown, resources);
	}
	private Style mkBlackOnWhite() {
		return new BWStyle();
	}
}
