package org.thibault.android.clock.binary;

import java.lang.reflect.Method;

import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.widget.*;

public class Widget extends AppWidgetProvider {
    
    public final static String CLOCK_START	= "BinaryClock.ClockStart";
    public final static String CLOCK_DELETE	= "BinaryClock.ClockDelete";
    public final static String CLOCK_STOP	= "BinaryClock.ClockStop";
    public final static String CLOCK_RESIZE = "BinaryClock.Resize";
    public final static String CLOCK_WIDGET_ID = "BinaryClock.WidgetId";
    public final static String CLOCK_WIDGET_IDS = "BinaryClock.WidgetIds";
    public final static String CLOCK_WIDGET_OPTIONS = "BinaryClock.WidgetOptions";
    public final static String CLOCK_WIDGET_WITH_TEXT = "CLOCK_WIDGET_WITH_TEXT";
    public final static String CLOCK_WIDGET_STYLE_ID = "CLOCK_WIDGET_STYLE_ID";
    public final static String CLOCK_SCREEN_DENSITY = "CLOCK_SCREEN_DENSITY";

	static void say(Context context, String msg){
		Log.d("BinaryClock.Widget",msg);
		//Toast t=Toast.makeText(context,msg,Toast.LENGTH_SHORT);
		//t.show();
	}
	
	@Override
	public void onAppWidgetOptionsChanged(Context context,
			AppWidgetManager appWidgetManager, int appWidgetId,
			Bundle newOptions) {
		int minWidth=newOptions.getInt(AppWidgetManager.OPTION_APPWIDGET_MIN_WIDTH);
		int minHeight=newOptions.getInt(AppWidgetManager.OPTION_APPWIDGET_MIN_HEIGHT);
		int maxWidth=newOptions.getInt(AppWidgetManager.OPTION_APPWIDGET_MAX_WIDTH);
		int maxHeight=newOptions.getInt(AppWidgetManager.OPTION_APPWIDGET_MAX_HEIGHT);
		say(context,"onAppWidgetOptionsChanged: ("+minWidth+"x"+minHeight+") to ("+maxWidth+"x"+maxHeight+")");

		Intent i = new Intent();
	    i.setClass(context, Updater.class);
	    i.setAction(CLOCK_RESIZE);
	    i.putExtra(CLOCK_WIDGET_ID, appWidgetId);
	    i.putExtra(CLOCK_WIDGET_OPTIONS, newOptions);

	    final float density = context.getResources().getDisplayMetrics().density;
		i.putExtra(CLOCK_SCREEN_DENSITY, density);
	    
		context.startService(i);
	}

	public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
		_onUpdate(context,appWidgetManager,appWidgetIds,false,-1);
	}
	public static void _onUpdate(Context context,
								 AppWidgetManager appWidgetManager, int[] appWidgetIds,
								 boolean withText, int styleId) {
		//say(context,"onUpdate: "+ appWidgetIds);
        Intent i = new Intent();
        i.setClass(context, Updater.class);
        i.setAction(CLOCK_START);
	    i.putExtra(CLOCK_WIDGET_IDS, appWidgetIds);
	    if (styleId>=0) {
		    i.putExtra(CLOCK_WIDGET_WITH_TEXT, withText);
	    	i.putExtra(CLOCK_WIDGET_STYLE_ID, styleId);
	    }

		final float density = context.getResources().getDisplayMetrics().density;
		i.putExtra(CLOCK_SCREEN_DENSITY, density);

	    try {
	    	Method getAppWidgetOptions=AppWidgetManager.class.getMethod("getAppWidgetOptions", int.class);
	    	for (int id: appWidgetIds)
	    		i.putExtra(CLOCK_WIDGET_OPTIONS+"_"+id,
	    				   (Bundle)getAppWidgetOptions.invoke(appWidgetManager, Integer.valueOf(id)));
	    } catch (Exception e) {
	    	say(context,"onUpdate(): Can't get widget options: API level too low, belike");
	    }
        context.startService(i);
	}
	
	public void onDeleted(Context context, int[] appWidgetIds) {
		_onDeleted(context,appWidgetIds);
	}
	
	public static void _onDeleted(Context context, int[] appWidgetIds) {
		say(context,"onDeleted");
        Intent i = new Intent();
        i.setClass(context, Updater.class);
        i.setAction(CLOCK_DELETE);
	    i.putExtra(CLOCK_WIDGET_IDS, appWidgetIds);
	    context.startService(i);
	}

	@Override  
    public void onDisabled(Context context)  
    {  
	say(context,"onDisabled");    Intent i = new Intent();
    i.setClass(context, Updater.class);
    i.setAction(CLOCK_STOP);
    context.startService(i);

    }
}
